/**
 * A circle with a radius.
 *
 * @author bsnider@georgefox.edu
 */
public class Circle extends Shape
{
    private double _radius;

    /**
     * Constructs a new circle with the specified radius.
     *
     * @param radius The radius of the new circle
     */
    public Circle(double radius)
    {
        _radius = radius;
    }

    /**
     * Constructs a new circle with a default radius of 1.0.
     */
    public Circle()
    {
        this(1.0);
    }

    /**
     * Gets the radius of this circle.
     *
     * @return The radius of this circle
     */
    public double getRadius()
    {
        return _radius;
    }

    /**
     * Sets the radius of this circle.
     *
     * @param newRadius The new radius of this circle
     */
    public void setRadius(double newRadius)
    {
        _radius = newRadius;
    }

    /**
     * Calculates the area of this circle.
     *
     * area = pi * r ^ 2
     *
     * Overrides the abstract calcArea() method from the Shape class,
     * and provides a concrete implementation specific to circles.
     *
     * @return The area of this circle
     */
    @Override
    public double calcArea()
    {
        return Math.PI * Math.pow(_radius, 2);
    }

    /**
     * Calculates the perimeter (circumference) of this circle.
     *
     * perimeter = 2 * pi * r;
     *
     * Overrides the abstract calcPerimeter() method from the Shape class,
     * and provides a concrete implementation specific to circles.
     *
     * @return The perimeter of this circle
     */
    @Override
    public double calcPerimeter()
    {
        return 2 * Math.PI * _radius;
    }

    /**
     * Calculates the circumference of this circle.
     *
     * A convenience method that simply calls the calcPerimeter() method,
     * since we like to refer to this measure as the circumference of a circle,
     * rather than the perimeter.
     *
     * @return The circumference of this circle.
     */
    public double calcCircumference()
    {
        return calcPerimeter();
    }

    /**
     * Returns a string representation of this circle.
     *
     * Overrides the toString() method from the Shape class,
     * but uses it, along with additional information from this circle,
     * to provide a string representation of this circle.
     *
     * @return A string representation of this circle.
     */
    @Override
    public String toString()
    {
        return super.toString() + String.format(", radius=%.5f", _radius);
    }
}
