/**
 * A rectangle with a length and a width.
 *
 * @author bsnider@georgefox.edu
 */
public class Rectangle extends Shape
{
    private double _length;
    private double _width;

    /**
     * Constructs a new rectangle with the specified length and width.
     *
     * @param length The length of the new rectangle.
     * @param width The width of the new rectangle.
     */
    public Rectangle(double length, double width)
    {
        _length = length;
        _width = width;
    }

    /**
     * Calculates the area of this rectangle.
     *
     * area = length * width
     *
     * Overrides the abstract calcArea() method from the Shape class,
     * and provides a concrete implementation specific to rectangles.
     *
     * @return The area of this rectangle.
     */
    @Override
    public double calcArea()
    {
        return _length * _width;
    }

    /**
     * Calculates the perimeter of this rectangle.
     *
     * perimeter = 2 * (length + width);
     *
     * Overrides the abstract calcPerimeter() method from the Shape class,
     * and provides a concrete implementation specific to rectangles.
     *
     * @return The perimeter of this rectangle
     */
    @Override
    public double calcPerimeter()
    {
        return 2 * (_length + _width);
    }

    /**
     * Gets the length of this rectangle.
     *
     * @return The length of this rectangle
     */
    public double getLength()
    {
        return _length;
    }

    /**
     * Sets the length of this rectangle.
     *
     * @param newLength The new length of this rectangle.
     */
    public void setLength(double newLength)
    {
        _length = newLength;
    }

    /**
     * Gets the width of this rectangle.
     *
     * @return The width of this rectangle
     */
    public double getWidth()
    {
        return _width;
    }

    /**
     * Sets the width of this rectangle.
     *
     * @param newWidth The new width of this rectangle
     */
    public void setWidth(double newWidth)
    {
        _width = newWidth;
    }

    /**
     * Returns a string representation of this rectangle.
     *
     * Overrides the toString() method from the Shape class,
     * but uses it, along with additional information from this rectangle,
     * to provide a string representation of this rectangle.
     *
     * @return A string representation of this rectangle.
     */
    @Override
    public String toString()
    {
        return String.format("%s, length=%.5f, width=%.5f",
            super.toString(), _length, _width);
    }
}
