/**
 * An abstract two-dimensional shape.
 *
 * A shape can calculate its area and perimeter, and provide a
 * string representation of itself.
 *
 * @author bsnider@georgefox.edu
 */
public abstract class Shape
{
    private static final double EPSILON = 1e-10;

    /**
     * Calculates the area of this shape.
     *
     * Abstract by design, as we are not able to implement the
     * area calculation for some arbitrary shape;
     * subclasses must override and implement.
     *
     * @return The area of this shape
     */
    public abstract double calcArea();

    /**
     * Calculates the perimeter of this shape.
     *
     * Abstract by design, as we are not able to implement the
     * perimeter calculation for some arbitrary shape;
     * subclasses must override and implement.
     *
     * @return The perimeter of this shape
     */
    public abstract double calcPerimeter();

    /**
     * Determines if this shape is equal to another shape.
     *
     * Shapes are considered equal if their areas are equal.
     * Concrete, as we are able to implement this method simply by
     * calling Shape methods.
     *
     * @param otherShape The other shape to test for equality
     * @return True if this shape is equal to the other shape, else false
     */
    public boolean equals(Shape otherShape)
    {
        return Math.abs(this.calcArea() - otherShape.calcArea()) < EPSILON;
    }

    /**
     * Returns a string representation of this shape.
     *
     * Concrete, as we are able to implement this method simply by
     * calling Shape methods.
     *
     * @return A string representation of this shape.
     */
    @Override
    public String toString()
    {
        return String.format("%s with area=%.5f",
            this.getClass().getName(), this.calcArea()
        );
    }
}
