/**
 * A square with a side length.
 *
 * A square is a special case of a rectangle, where the length and width
 * are the same.
 * Note that we don't need to override most methods from class Rectangle,
 * as that inherited implementation works just fine for Squares too.
 * However, note that this means that, as implemented, we're limited to
 * getWidth() and getLength() methods, unless we add a new getSideLength()
 * convenience method to match our constructor.
 * In that case, we'd simply return the return value of getLength(), since
 * we're not actually storing the side length in a private attribute here.
 *
 * @author bsnider@georgefox.edu
 */
public class Square extends Rectangle
{
    /**
     * Constructs a new square with the specified side length.
     *
     * Since constructors are not members and therefore not inherited,
     * we need to implement a constructor here.
     * There is no one-argument constructor in the Rectangle class,
     * so we simply call the two-argument constructor and provide the
     * same value for both the length and the width.
     * This allows the inherited (but not overridden) methods from class
     * Rectangle to work correctly.
     *
     * @param sideLength The side length of the new square
     */
    public Square(double sideLength)
    {
        super(sideLength, sideLength);
    }

    /**
     * Returns a string representation of this rectangle.
     *
     * Overrides the toString() method from the Rectangle class.
     * Since we don't want to include mention of individual length and width
     * properties, we can't use the Rectangle's toString() method here.
     * Instead, we provide our own implementation that follows the same format.
     *
     * @return A string representation of this rectangle.
     */
    @Override
    public String toString()
    {
        return String.format("%s with area=%.5f, side length=%.5f",
                getClass().getName(), calcArea(), getLength());
    }
}
